/**
* Keyboard input utility to overcome Adobe's marvellous security restrictions
* Outstanding bug with keys sticking that occurs once in a blue moon - still haven't caught it yet
*
* @author Aaron Steed, roboacid.com
* @version 1.1
*
* Adapted from http://www.kirupa.com/forum/showthread.php?p=2098269
*
* Comments from original follow:
*
* The Key class recreates functionality of
* Key.isDown of ActionScript 1 and 2. Before using
* Key.isDown, you first need to initialize the
* Key class with a reference to the stage using
* its Key.initialize() method. For key
* codes use the flash.ui.Keyboard class.
*
* Usage:
* Key.initialize(stage);
* if (Key.isDown(Keyboard.LEFT)) {
*    // Left key is being pressed
* }
*/

package com.robotacid.game {
    import flash.display.Stage;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	public class Key {
		private static var initialized:Boolean = false;  // marks whether or not the class has been initialized
        private static var keysDown:Array = [];  // stores key codes of all keys pressed
		public static var custom:Array; // list of customised keys
		public static var reserved:Array = []; // list of reserved keys
		public static var stage:Stage;
		public static var keys_pressed:int = 0;
		
		public function Key() {
		}
        /*
        * Initializes the key class creating assigning event
        * handlers to capture necessary key events from the stage
		*
		* optional custom_keys is an array of key codes referring to
		* user definable keys
        */
        public static function init(_stage:Stage):void {
            if (!initialized) {
                stage = _stage;
				
				// assign listeners for key presses and deactivation of the player
                stage.addEventListener(KeyboardEvent.KEY_DOWN, keyPressed);
                stage.addEventListener(KeyboardEvent.KEY_UP, keyReleased);
                stage.addEventListener(Event.DEACTIVATE, clearKeys);
				
                // mark initialization as true so redundant
                // calls do not reassign the event handlers
                initialized = true;
            }
        }
		
        /**
        * Returns true or false if the key represented by the
        * custom key index is being pressed
        */
        public static function customDown(index:int):Boolean {
            if (!initialized) {
                // throw an error if isDown is used
                // prior to Key class initialization
                throw new Error("Key class has yet been initialized.");
            }
            return custom != null && Boolean(keysDown[custom[index]]);
        }
		
        /**
        * Returns true or false if the key represented by the
        * keyCode passed is being pressed
        */
        public static function isDown(keyCode:int):Boolean {
            if (!initialized) {
                // throw an error if isDown is used
                // prior to Key class initialization
                throw new Error("Key class has yet been initialized.");
            }
            return Boolean(keysDown[keyCode]);
        }

        /**
        * Event handler for capturing keys being pressed
        */
        private static function keyPressed(event:KeyboardEvent):void {
            // create a property in keysDown with the name of the keyCode
			if(!Boolean(keysDown[event.keyCode])) keys_pressed++;
            keysDown[event.keyCode] = true;
        }

        /**
        * Event handler for capturing keys being released
        */
        private static function keyReleased(event:KeyboardEvent):void {
            keysDown[event.keyCode] = false;
			keys_pressed--;
        }

        /**
        * Event handler for Flash Player deactivation
        */
        private static function clearKeys(event:Event):void {
            // clear all keys in keysDown since the player cannot
            // detect keys being pressed or released when not focused
            keysDown = [];
			keys_pressed = 0;
        }
		public static function forceClearKeys():void{
            keysDown = [];
			keys_pressed = 0;
		}
		
		/*
		 * Return a string representing a key pressed
		 * a 3 letter string is returned for special characters
		 *
		 */
		
		public static function keyString(key_code:uint):String{
			switch(key_code){
				case Keyboard.BACKSPACE:
					return "bsp";
				case Keyboard.CAPS_LOCK:
					return "cpl";
				case Keyboard.CONTROL:
					return "ctr";
				case Keyboard.DELETE:
					return "del";
				case Keyboard.DOWN:
					return "dwn";
				case Keyboard.END:
					return "end";
				case Keyboard.ENTER:
					return "ent";
				case Keyboard.ESCAPE:
					return "esc";
				case Keyboard.HOME:
					return "hom";
				case Keyboard.INSERT:
					return "ins";
				case Keyboard.LEFT:
					return "lft";
				case Keyboard.PAGE_DOWN:
					return "pgd";
				case Keyboard.PAGE_UP:
					return "pgu";
				case Keyboard.RIGHT:
					return "rgt";
				case Keyboard.SHIFT:
					return "sht";
				case Keyboard.SPACE:
					return "spc";
				case Keyboard.TAB:
					return "tab";
				case Keyboard.UP:
					return "up";
				case 186:
					return ":";
				case 188:
					return ".";
				case 190:
					return ",";
				case 191:
					return "?";
				case 109:
					return "n -";
				case 107:
					return "n +";
				case 187:
					return "+";
				case 189:
					return "-";
				case 222:
					return "'";
				default:
					if(key_code >= 96 && key_code <= 105){
						return "n "+String.fromCharCode(key_code-48);
					} else {
						return String.fromCharCode(key_code);
					}
			}
			return "";
		}
	}
}
